/**
 * 
 */
package ru.urvanov.aliceinwonderland;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.IOException;
import java.util.ResourceBundle;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ru.urvanov.aliceinwonderland.localization.AliceInWonderlandResourceBundle;
import ru.urvanov.aliceinwonderland.localization.StringConstants;



/**
 * @author Урванов Фёдор Владиславович
 *
 */
public class AliceInWonderlandMain extends JPanel
        implements MouseListener, MouseMotionListener {
    private Image imgMain;
    private Image imgHalfCup;
    private Image imgMessage;
    private ImageIcon icoClock;
    private MediaTracker tracker;
    private javax.swing.Timer tmrWait;
    private boolean bLoaded;
    private JLabel lblClock;
    private WavPlayer wavPlayer;
    public static final int TIP_HALF_CUP = 1;
    public static final int TIP_NEW_CUP_MESSAGE = 2;
    public static final int TIP_FIVE_O_CLOCK = 3;
    public static final int TIP_AUTHOR = 4;
    private int nShowTip = 0;
    private ResourceBundle resourceBundle;
    private Font fntMessage;
    public static final int MESSAGE_LEFT = 100;
    public static final int MESSAGE_TOP = 260;
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    public static void main(String[] args) {
        JFrame frame = new JFrame();
        frame.getContentPane().add(new AliceInWonderlandMain());
        frame.setLocationByPlatform(true);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setTitle("Alice in Wonderland");
        frame.setVisible(true);
        frame.pack();
    }
    
    public AliceInWonderlandMain() {
        init();
    }

    public void init()
	{
        setPreferredSize(new Dimension(640, 480));
		bLoaded=false;
		fntMessage=new Font("Monospaced", Font.BOLD,15);
		try {
		    imgMain=ImageIO.read(AliceInWonderlandMain.class.getResource("aliceinwonderland.png"));
		    imgHalfCup=ImageIO.read(AliceInWonderlandMain.class.getResource("halfcup.png"));
		    imgMessage=ImageIO.read(AliceInWonderlandMain.class.getResource("message.png"));
		    icoClock=new ImageIcon(AliceInWonderlandMain.class.getResource("clock.gif"));
		} catch (IOException e) {
		    e.printStackTrace();
		}
		resourceBundle=
			AliceInWonderlandResourceBundle.getBundle("ru.urvanov.aliceinwonderland.localization.AliceInWonderlandResourceBundle");
		wavPlayer=new WavPlayer("clock.wav");
		wavPlayer.start();
		tracker=new MediaTracker(this);
		
		
		tracker.addImage(icoClock.getImage(),0);
		try {
			tracker.waitForID(0);
			tracker.removeImage(icoClock.getImage());
		} catch (InterruptedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		lblClock=new JLabel(icoClock);
		add(lblClock);
		
		tracker.addImage(imgMain, 1);
		tracker.addImage(imgHalfCup,1);
		tracker.addImage(imgMessage,1);
		ActionListener taskTimer = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
					try {
						
		            	if (tracker.waitForID(1,10))
		            	{
		            		tracker.removeImage(imgMain);
		            		tracker.removeImage(imgHalfCup);
		            		tracker.removeImage(imgMessage);
		            		remove(lblClock);
		            		bLoaded=true;
		            		Graphics g=getGraphics();
		            		update(g);
		            		tmrWait.stop(); 
		            	}
		            	
		        	} catch (InterruptedException e) {}
		        	
			}
		};
		
		tmrWait=new javax.swing.Timer(50, taskTimer);
        tmrWait.start();
		addMouseListener(this);
		addMouseMotionListener(this);
	}

    public void start() {
        repaint();
    }

    public void stop() {
        invalidate();
    }

    public void destroy() {
        wavPlayer.sendCloseMessage();
        try {
            wavPlayer.join();
        } catch (InterruptedException interruptedException) {
            interruptedException.printStackTrace();
        }
    }

    public void paint(Graphics p_g) {
        Graphics2D g = (Graphics2D) p_g;
        if (bLoaded) {
            g.drawImage(imgMain, 0, 0, null);
            switch (nShowTip) {
            case TIP_HALF_CUP:
                g.drawImage(imgHalfCup, 220, 190, null);
                break;
            case TIP_NEW_CUP_MESSAGE:
                g.drawImage(imgMessage, MESSAGE_LEFT, MESSAGE_TOP, null);
                g.setFont(fntMessage);
                g.drawString(resourceBundle
                        .getString(StringConstants.I_WANT_NEW_CUP), 200, 380);
                break;
            case TIP_FIVE_O_CLOCK:
                g.drawImage(imgMessage, MESSAGE_LEFT, MESSAGE_TOP, null);
                g.setFont(fntMessage);
                g.drawString(
                        resourceBundle.getString(StringConstants.FIVE_O_CLOCK),
                        200, 380);
                break;
            case TIP_AUTHOR:
                g.drawImage(imgMessage, MESSAGE_LEFT, MESSAGE_TOP, null);
                g.setFont(fntMessage);
                g.drawString(resourceBundle.getString(StringConstants.AUTHOR),
                        180, 360);
                g.drawString(resourceBundle.getString(StringConstants.SITE),
                        180, 380);
                g.drawString(resourceBundle.getString(StringConstants.EMAIL),
                        180, 400);
                break;
            }

        } else {
            super.paint(g);
        }
    }

    @Override
    public void mouseClicked(MouseEvent arg0) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseEntered(MouseEvent arg0) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseExited(MouseEvent arg0) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mousePressed(MouseEvent arg0) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseReleased(MouseEvent arg0) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseDragged(MouseEvent arg0) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseMoved(MouseEvent p_me) {
        // TODO Auto-generated method stub
        int nLastShowTip = nShowTip;
        int x = p_me.getX();
        int y = p_me.getY();
        if ((210 <= x) && (x <= 240) && (180 <= y) && (y <= 210)) {
            nShowTip = TIP_HALF_CUP;
        } else if ((90 <= x) && (x <= 220) && (20 <= y) && (y <= 180)) {
            nShowTip = TIP_NEW_CUP_MESSAGE;
        } else if ((440 <= x) && (x <= 560) && (40 <= y) && (y <= 183)) {
            nShowTip = TIP_FIVE_O_CLOCK;
        } else if ((x <= 70) && (y <= 70)) {
            nShowTip = TIP_AUTHOR;
        } else {
            nShowTip = 0;
        }
        if (nLastShowTip != nShowTip) {
            repaint();
        }
    }
}
