package ru.urvanov.aliceinwonderland;

import java.io.File; 
import java.io.IOException; 
import java.net.URL;

import javax.sound.sampled.AudioFormat; 
import javax.sound.sampled.AudioInputStream; 
import javax.sound.sampled.AudioSystem; 
import javax.sound.sampled.DataLine; 
import javax.sound.sampled.FloatControl; 
import javax.sound.sampled.LineUnavailableException; 
import javax.sound.sampled.SourceDataLine; 
import javax.sound.sampled.UnsupportedAudioFileException; 
 
/**
 * 
 *  new WavPlayer("data/sounds/button.wav").start();
 * @author fedya
 *
 */
public class WavPlayer extends Thread 
{
	private boolean bClosing=false;
	
    private String filename;
 
    private Position curPosition;
 
    private final int EXTERNAL_BUFFER_SIZE = 10240; // 128Kb 
 
    enum Position { 
        LEFT, RIGHT, NORMAL
    };
    
    public WavPlayer(String wavfile) { 
        filename = wavfile;
        curPosition = Position.NORMAL;
    } 
    public WavPlayer(String wavfile, Position p) { 
        filename = wavfile;
        curPosition = p;
    } 

    
    
   
    public void run() 
    { 
    	
    	
    	URL soundurl=null;
    	File soundFile=null;
   		soundurl= this.getClass().getResource(filename);

 
        AudioInputStream audioInputStream = null;
        try { 
        	if (soundurl!=null)
        	{
        		audioInputStream = AudioSystem.getAudioInputStream(soundurl);
        	}
        	else
        	{
        		audioInputStream = AudioSystem.getAudioInputStream(soundFile);
        	}
        } 
        catch (UnsupportedAudioFileException e1) 
        { 
            e1.printStackTrace();
            return;
        } 
        catch (IOException e1) 
        { 
            e1.printStackTrace();
            return;
        } 
 
        AudioFormat format = audioInputStream.getFormat();
        SourceDataLine auline = null;
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
 
        try 
        { 
            auline = (SourceDataLine) AudioSystem.getLine(info);
            auline.open(format);
        } 
        catch (LineUnavailableException e) 
        { 
            e.printStackTrace();
            return;
        } 
        catch (Exception e) 
        { 
            e.printStackTrace();
            return;
        } 
 
        if (auline.isControlSupported(FloatControl.Type.PAN)) { 
            FloatControl pan = (FloatControl) auline
                    .getControl(FloatControl.Type.PAN);
            if (curPosition == Position.RIGHT) 
                pan.setValue(1.0f);
            else if (curPosition == Position.LEFT) 
                pan.setValue(-1.0f);
        } 

        auline.start();
        int nBytesRead = 0;
        byte[] abData=null;
        byte[] abBuff=new byte[EXTERNAL_BUFFER_SIZE];
        try {
        	int nCurrentSize=0;
        	int abDataLength=0;
        	while (nBytesRead != -1) 
        	{        		
    			nBytesRead = audioInputStream.read(abBuff, 0, abBuff.length);
    			if (nBytesRead >= 0) 
    			{
    				nCurrentSize+=nBytesRead;
    				byte[] abBuff2=new byte[nCurrentSize];
    				if (abDataLength>0)
    				{
    					for (int n=0; n<abData.length;n++)
    					{
    						abBuff2[n]=abData[n];
    					}
    				}
    				for (int n=0;n<nBytesRead;n++)
    				{
    					abBuff2[n+abDataLength]=abBuff[n];
    				}
    				abData=abBuff2;
    				abDataLength=abData.length;
    			}
    		
    		}
        	while (!bClosing)
        	{
        		for (int n=0;((n<abData.length)&&(!bClosing));n+=EXTERNAL_BUFFER_SIZE)
        		{
        			auline.write(abData, 0, abData.length);
        			
        		}
         	}
            
        } 
        catch (IOException e) 
        { 
            e.printStackTrace();
            return;
        } 
        finally { 
            auline.drain();
            auline.close();
        } 

    }
    
    public void sendCloseMessage()
    {
    	bClosing=true;
    }

} 
 